@@ -18,8 +18,6 @@
 #include <dirent.h>
 #include "cl_v4l2_handler.h"
 
-static bool type_loaded = false;
-
 // PKG_ADD: autoload ("__v4l2_handler_open__", which ("__v4l2_handler__.oct"));
 // PKG_DEL: autoload ("__v4l2_handler_open__", which ("__v4l2_handler__.oct"), "remove");
 DEFUN_DLD(__v4l2_handler_open__, args, nargout,
@@ -38,18 +36,12 @@ Creates an instance of v4l2_handler for a v4l2 device and opens it.\n\
       return retval;
     }
 
-  if (!type_loaded)
-    {
-      v4l2_handler::register_type();
-      type_loaded = true;
-    }
   string device = args(0).string_value ();
-  if (! error_state)
-    {
-      v4l2_handler *h = new v4l2_handler ();
-      h->open (device.c_str ());
-      retval.append (octave_value (h));
-    }
+
+  v4l2_handler *h = new v4l2_handler ();
+  h->open (device.c_str (), false);
+  retval.append (octave_value (h));
+
   return retval;
 }
 
@@ -85,7 +77,7 @@ DEFUN_DLD(__v4l2_handler_enuminput__, args, nargout,
           "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {@var{inputs} = } __v4l2_handler_enuminput__ (@var{h})\n\
 Enumerate video inputs from v4l2_handler @var{h}.\n\
-Returns a struct with informations for all avaliable v4l2 inputs.\n\
+Returns a struct with information for all available v4l2 inputs.\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -149,11 +141,13 @@ Select video input @var{n} from v4l2_handler @var{h}.\n\
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
     {
-      int num = args(1).int_value ();
-      if (!error_state)
-        imgh->s_input (num);
-      else
+      if (! args(1).isnumeric())
         error("N has to be a integer selecting the desired video input, starting from  0.");
+      else
+        {
+          int num = args(1).int_value ();
+          imgh->s_input (num);
+        }
     }
   return retval;
 }
@@ -165,7 +159,7 @@ DEFUN_DLD(__v4l2_handler_enum_fmt__, args, nargout,
           "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {@var{formats} = } __v4l2_handler_enum_fmt__ (@var{h})\n\
 Enumerate image formats from v4l2_handler @var{h}.\n\
-Returns a struct with informations for all avaliable v4l2 formats.\n\
+Returns a struct with information for all available v4l2 formats.\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -232,15 +226,16 @@ Return a Nx2 matrix with numerator, denominator.\n\
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
     {
-      Matrix s = args(1).matrix_value ();
-      unsigned int width = s(0);
-      unsigned int height = s(1);
-      if (error_state)
+      if (!args (1).is_matrix_type())
+        print_usage();
+      else
         {
-          print_usage();
+          Matrix s = args(1).matrix_value ();
+          unsigned int width = s(0);
+          unsigned int height = s(1);
+          string pixel_format = args(2).string_value ();
+          retval = octave_value(imgh->enum_frameintervals (pixel_format, width, height));
         }
-      string pixel_format = args(2).string_value ();
-      retval = octave_value(imgh->enum_frameintervals (pixel_format, width, height));
     }
   return retval;
 }
@@ -336,6 +331,11 @@ Set format @var{fmt}, @var{size} (V4L2_FIELD_INTERLACED).\n\
       print_usage ();
       return retval;
     }
+  if (!args (1).is_string() || !args (2).is_matrix_type())
+    {
+      print_usage();
+      return retval;
+    }
 
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
@@ -344,10 +344,8 @@ Set format @var{fmt}, @var{size} (V4L2_FIELD_INTERLACED).\n\
       Matrix s = args(2).matrix_value ();
       unsigned int xres = s(0);
       unsigned int yres = s(1);
-      if (! error_state)
-        {
-          imgh->s_fmt (fmt, xres, yres);
-        }
+
+      imgh->s_fmt (fmt, xres, yres);
     }
   return retval;
 }
@@ -398,15 +396,17 @@ Use the field id from __v4l2_handler_queryctrl__.\n\
       print_usage ();
       return retval;
     }
+  if (!args (1).isnumeric())
+    {
+      error("ID has to be an integer value");
+      return retval;
+    }
 
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
     {
       unsigned int id = args(1).int_value ();
-      if (!error_state)
-        retval = octave_value(imgh->g_ctrl (id));
-      else
-        error("ID has to be an integer value");
+      retval = octave_value(imgh->g_ctrl (id));
     }
   return retval;
 }
@@ -429,16 +429,17 @@ Use the field id from __v4l2_handler_queryctrl__.\n\
       print_usage ();
       return retval;
     }
-
+  if (!args (1).isnumeric() || !args (2).isnumeric())
+    {
+      error("ID and VALUE has to be integer values");
+      return retval;
+    }
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
     {
       unsigned int id = args(1).int_value ();
       unsigned int value = args(2).int_value ();
-      if (!error_state)
-        imgh->s_ctrl (id, value);
-      else
-        error("ID and VALUE has to be integer values");
+      imgh->s_ctrl (id, value);
     }
   return retval;
 }
@@ -485,15 +486,16 @@ Start streaming with @var{n} buffers. It is recommended to use at least 2 buffer
       print_usage ();
       return retval;
     }
+  if (!args (1).isnumeric())
+    {
+      return retval;
+    }
 
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
     {
       unsigned int n_buffers = args(1).int_value ();
-      if (! error_state)
-        {
-          imgh->streamon (n_buffers);
-        }
+      imgh->streamon (n_buffers);
     }
   return retval;
 }
@@ -515,6 +517,10 @@ Get a snapshot from v4l2_handler @var{h}\n\
       print_usage ();
       return retval;
     }
+  if (nargin > 1 && !args (1).isnumeric())
+    {
+      return retval;
+    }
 
   v4l2_handler* imgh = get_v4l2_handler_from_ov (args(0));
   if (imgh)
@@ -522,10 +528,7 @@ Get a snapshot from v4l2_handler @var{h}\n\
       int preview = 0;
       if (nargin==2)
         preview = args(1).int_value ();
-      if (!error_state)
-        {
-          retval = imgh->capture (nargout, preview);
-        }
+      retval = imgh->capture (nargout, preview);
     }
   return retval;
 }
@@ -558,7 +561,6 @@ Use '$ v4l2-ctl --list-devices' for more details.\n\
   DIR *dp;
   struct dirent *ep;
   dev_vec files;
-
   dp = opendir("/dev");
   if (dp == NULL) {
     error ("Couldn't open /dev/ directory");
@@ -573,11 +575,18 @@ Use '$ v4l2-ctl --list-devices' for more details.\n\
   for (dev_vec::iterator iter = files.begin();
       iter != files.end(); ++iter)
     {
+      //printf ("trying '%s'...\n", iter->c_str());
+      //fflush(stdout);
       v4l2_handler h;
-      h.open(iter->c_str());
-      octave_scalar_map caps = h.querycap ().scalar_map_value();
-      caps.assign ("device", *iter);
-      retval.assign(i++, caps);
+      octave_scalar_map caps = h.open(iter->c_str(), true);
+
+      if (! h.is_meta_capture ())
+      {
+        caps.assign ("device", *iter);
+        retval.assign(i++, caps);
+      }
+      //else
+        //printf ("INFO: list device ignores metadata interface '%s'...\n", iter->c_str());
     }
   return octave_value (retval);
 }
