@@ -145,12 +145,19 @@ static unsigned int v4l2_format_code(const char *name)
 
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(v4l2_handler, "v4l2_handler", "v4l2_handler");
 
+bool v4l2_handler::type_loaded = false;
+
 v4l2_handler::v4l2_handler ()
   : octave_base_value(),
     fd(-1), n_buffer(0), buffers(0), streaming(0),
-    preview_window(0)
+    preview_window(0), _is_video_capture (0), _is_meta_capture(0)
 {
-  //octave_stdout << "v4l2_handler C'Tor " << endl;
+  //octave_stdout << "v4l2_handler C'Tor, type_loaded = " << type_loaded << endl;
+  if (!type_loaded)
+    {
+      type_loaded = true;
+      v4l2_handler::register_type();
+    }
 }
 
 v4l2_handler::v4l2_handler (const v4l2_handler& m)
@@ -199,9 +206,10 @@ v4l2_handler::xioctl_name (int fh, unsigned long int request, void *arg, const c
     }
 }
 
-void
-v4l2_handler::open (string d)
+octave_scalar_map
+v4l2_handler::open (string d, bool quiet)
 {
+  octave_scalar_map ret;
   fd = v4l2_open(d.c_str(), O_RDWR | O_NONBLOCK, 0);
   if (fd < 0)
     {
@@ -209,12 +217,62 @@ v4l2_handler::open (string d)
     }
   else
     {
-      dev = d;
+      dev = d; // store device path for later info output
+	  ret = querycap ().scalar_map_value ();
+
+	  if (!quiet && is_meta_capture ())
+	    warning ("Device '%s' is a metadata interface device", d.c_str());
     }
+  return ret;
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-querycap.html
+ * Convert bitfield to scalar struct
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-querycap.html#device-capabilities
+ * See also videodev2.h
+ */
+octave_scalar_map
+v4l2_handler::expand_cap (unsigned int cap)
+{
+  octave_scalar_map ret;
+  ret.assign ("raw", cap);
+  #define CHECK_DEVICE_CAPABILITIES_FIELD(X) ret.assign (#X, (bool)(cap & X))
+
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_CAPTURE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_OUTPUT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_OVERLAY);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VBI_CAPTURE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VBI_OUTPUT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_SLICED_VBI_CAPTURE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_SLICED_VBI_OUTPUT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_RDS_CAPTURE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_OUTPUT_OVERLAY);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_HW_FREQ_SEEK);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_RDS_OUTPUT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_CAPTURE_MPLANE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_OUTPUT_MPLANE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_M2M_MPLANE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_M2M);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_TUNER);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_AUDIO);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_RADIO);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_MODULATOR);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_SDR_CAPTURE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_EXT_PIX_FORMAT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_SDR_OUTPUT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_META_CAPTURE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_READWRITE);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_STREAMING);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_META_OUTPUT);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_TOUCH);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_IO_MC);
+  CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_DEVICE_CAPS);
+
+  return ret;
+}
+
+/*!
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-querycap.html
  * v4l2-ctl -D
  * \return octave_scalar_map with device capabilities
  */
@@ -226,17 +284,31 @@ v4l2_handler::querycap ()
   xioctl (fd, VIDIOC_QUERYCAP, &cap);
 
   octave_scalar_map st;
-  if (!error_state)
-    {
-      st.assign ("driver",    std::string((const char*)cap.driver));
-      st.assign ("card",      std::string((const char*)cap.card));
-      st.assign ("bus_info",  std::string((const char*)cap.bus_info));
 
-      char tmp[15];
-      snprintf (tmp, 15, "%u.%u.%u", (cap.version >> 16) & 0xFF, (cap.version >> 8) & 0xFF, cap.version & 0xFF);
-      st.assign ("version",   std::string(tmp));
-      st.assign ("capabilities", (unsigned int)(cap.capabilities));
-    }
+  st.assign ("driver",    std::string((const char*)cap.driver));
+  st.assign ("card",      std::string((const char*)cap.card));
+  st.assign ("bus_info",  std::string((const char*)cap.bus_info));
+
+  char tmp[15];
+  snprintf (tmp, 15, "%u.%u.%u", (cap.version >> 16) & 0xFF, (cap.version >> 8) & 0xFF, cap.version & 0xFF);
+  st.assign ("version", std::string(tmp));
+
+  st.assign ("capabilities", expand_cap (cap.capabilities));
+
+  // The driver fills the device_caps field. This capability can only
+  // appear in the capabilities field and never in the device_caps field.
+  if (cap.capabilities & V4L2_CAP_DEVICE_CAPS)
+  {
+    st.assign ("device_caps", expand_cap (cap.device_caps));
+    _is_video_capture = cap.device_caps & V4L2_CAP_VIDEO_CAPTURE;
+    _is_meta_capture  = cap.device_caps & V4L2_CAP_META_CAPTURE;
+  }
+  else
+  {
+    _is_video_capture = cap.capabilities & V4L2_CAP_VIDEO_CAPTURE;
+    _is_meta_capture  = cap.capabilities & V4L2_CAP_META_CAPTURE;
+  }
+
   return octave_value (st);
 }
 
@@ -407,19 +479,16 @@ v4l2_handler::g_parm ()
   CLEAR(sparam);
   sparam.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
   xioctl(fd, VIDIOC_G_PARM, &sparam);
-  if(!error_state)
+  if(sparam.parm.capture.capability & V4L2_CAP_TIMEPERFRAME)
     {
-      if(sparam.parm.capture.capability & V4L2_CAP_TIMEPERFRAME)
-        {
-          const struct v4l2_fract &tf = sparam.parm.capture.timeperframe;
-          ret(0) = tf.numerator;
-          ret(1) = tf.denominator;
-        }
-      else
-        {
-          warning("v4l2_handler::g_parm: V4L2_CAP_TIMEPERFRAME is not supported");
-          return Matrix(0,0);
-        }
+      const struct v4l2_fract &tf = sparam.parm.capture.timeperframe;
+      ret(0) = tf.numerator;
+      ret(1) = tf.denominator;
+    }
+  else
+    {
+      warning("v4l2_handler::g_parm: V4L2_CAP_TIMEPERFRAME is not supported");
+      return Matrix(0,0);
     }
   return ret;
 }
@@ -628,13 +697,13 @@ v4l2_handler::g_fmt ()
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-reqbufs.html
+ * https://www.kernel.org/doc/html/v4.9/media/uapi/v4l/vidioc-reqbufs.html
  * \param n number of buffers to initiate. A count value of zero frees all buffers.
  */
 void
 v4l2_handler::reqbufs (unsigned int n)
 {
-  if (fd>=0)
+  if (fd >= 0 && (n > 0 || n_buffer > 0))
     {
       struct v4l2_requestbuffers req;
       CLEAR(req);
@@ -642,7 +711,7 @@ v4l2_handler::reqbufs (unsigned int n)
       req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
       req.memory = V4L2_MEMORY_MMAP;
       xioctl(fd, VIDIOC_REQBUFS, &req);
-      if (req.count<n)
+      if (req.count < n)
         error("v4l2_handler::reqbufs: VIDIOC_REQBUFS: running out of free memory\n");
       n_buffer = req.count;
     }
@@ -721,15 +790,21 @@ v4l2_handler::capture (int nargout, int preview)
       FD_ZERO(&fds);
       FD_SET(fd, &fds);
 
-      // 2s Timeout TODO: make this configurable
-      tv.tv_sec = 2;
+      // 3s Timeout TODO: make this configurable
+      tv.tv_sec = 3;
       tv.tv_usec = 0;
       r = select(fd + 1, &fds, NULL, NULL, &tv);
     }
   while ((r == -1 && (errno == EINTR)));
   if (r == -1)
     {
-      error("v4l2_handler::capture: Select failed.");
+      error("v4l2_handler::capture: Select returned code %i (%s)", errno, strerror(errno));
+      return octave_value();
+    }
+
+  if (r == 0)
+    {
+      error("v4l2_handler::capture: Select timeout");
       return octave_value();
     }
 
@@ -979,7 +1054,7 @@ v4l2_handler::capture_to_ppm (const char *fn)
       error("v4l2_handler::capture_to_ppm: Cannot open file '%s'", fn);
     }
   fprintf (fout, "P6\n%d %d 255\n",
-           img.dim2(), img.dim3());
+           (int)img.dim2(), (int)img.dim3());
   fwrite (p, img.numel(), 1, fout);
   fclose (fout);
 }
@@ -1012,8 +1087,7 @@ v4l2_handler::streamon (unsigned int n)
       enum   v4l2_buf_type type;
       type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
       xioctl(fd, VIDIOC_STREAMON, &type);
-      if (!error_state)
-        streaming = 1;
+      streaming = 1;
     }
 }
 
@@ -1066,13 +1140,6 @@ v4l2_handler::close ()
 v4l2_handler*
 get_v4l2_handler_from_ov (octave_value ov)
 {
-  static bool type_loaded = false;
-  if (!type_loaded)
-    {
-      v4l2_handler::register_type();
-      type_loaded = true;
-    }
-
   if (ov.type_id() != v4l2_handler::static_type_id())
     {
       error("get_v4l2_handler_from_ov: Not a valid v4l2_handler");
